@@ -83,7 +83,7 @@ sub INIT_INSTANCE {
       # obscure some part of the window. se we have to do lots of unneessary refreshes :(
       $self->{window}->clear_area ($_[1]->area->values);
       $self->draw_drag_rect ($_[1]->area);
-      1 
+      1
    });
    $self->signal_connect (configure_event => sub { $_[0]->do_configure ($_[1]); 0 });
    $self->signal_connect (key_press_event => sub { $_[0]->handle_key ($_[1]->keyval, $_[1]->state) });
@@ -106,6 +106,7 @@ sub INIT_INSTANCE {
    #$self->set_resize_mode ("immediate");
 
    $self->{interp} = 'bilinear';
+   $self->{maxpect} = 1;
 
    $self->pop_composite_child;
 
@@ -133,16 +134,16 @@ sub kill_player {
 
    if ($self->{player_pid} > 0) {
 
-      if ($self->{mplayer_fh}) {
+      if ($self->{mpv_fh}) {
          local $SIG{PIPE} = 'IGNORE';
-         print {$self->{mplayer_fh}} "quit\n";
-         delete $self->{mplayer_fh};
+         print {$self->{mpv_fh}} "quit\n";
+         delete $self->{mpv_fh};
       } else {
          kill INT  => $self->{player_pid};
          kill TERM => $self->{player_pid};
       }
       waitpid delete $self->{player_pid}, 0;
-      (delete $self->{mplayer_box})->destroy if $self->{mplayer_box};
+      (delete $self->{mpv_box})->destroy if $self->{mpv_box};
    }
 }
 
@@ -296,7 +297,21 @@ sub load_image {
       my $moviedir = dir_is_movie $path;
 
       my $image;
-      my $type = Gtk2::CV::magic_mime $path;
+      my $type;
+
+      # magic/file is no longer very reliable
+      if (open my $fh, "<:raw", $path) {
+         if (sysread $fh, my $buf, 64) {
+            unless (defined ($type = Gtk2::CV::filetype $buf)) {
+               if ($buf =~ /^....ftypavif/) {
+                  $type = "image/avif";
+               }
+               #elsif ($buf =~ /^\xff\xd8\xff\xe0\x00.JFIF\x00\x01[\x00\x01\x02]/) { $type = "image/jpeg"; # not the only possible format
+            }
+         }
+      }
+
+      $type ||= Gtk2::CV::magic_mime $path;
 
       if ($type =~ /^application\//) {
          my $magic = Gtk2::CV::magic $path;
@@ -337,6 +352,9 @@ sub load_image {
       } elsif ($type eq "image/jpeg") {
          $image = Gtk2::CV::load_jpeg $path;
 
+      } elsif ($type eq "image/jxl") {
+         $image = Gtk2::CV::load_jxl $path;
+
       } elsif ($type eq "image/webp") {
          $image = Gtk2::CV::load_webp $path;
 
@@ -355,7 +373,13 @@ sub load_image {
       } elsif ($type =~ /^image\//) {
          open my $fh, "<", $path
             or die "$path: $!";
-         my $loader = new Gtk2::Gdk::PixbufLoader;
+
+         # avif fails autodetection
+         my $loader =
+            $type eq "image/avif"
+               ? new_with_type Gtk2::Gdk::PixbufLoader "heif/avif"
+               : new           Gtk2::Gdk::PixbufLoader;
+
          local $/; $loader->write (<$fh>);
          $loader->close;
          $image = $loader->get_pixbuf;
@@ -369,7 +393,7 @@ sub load_image {
 #            open STDOUT, ">/dev/null";
 #            open STDERR, ">&2";
 
-               my $player = $ENV{CV_AUDIO_PLAYER} || $Gtk2::CV::MPLAYER;
+               my $player = $ENV{CV_AUDIO_PLAYER} || $Gtk2::CV::MPV;
                $path = "./$path" if $path =~ /^-/;
                exec "$player \Q$path";
                POSIX::_exit 0;
@@ -439,15 +463,17 @@ sub load_image {
          } else {
             no integer;
 
-            my $mplayer = qx{LC_ALL=C exec \Q$Gtk2::CV::MPLAYER\E </dev/null 2>/dev/null -really-quiet -noconfig all -noautosub -nofontconfig -sub /dev/null -sub-fuzziness 0 -cache-min 0 -input nodefault-bindings:conf=/dev/null -identify -vo null -ao null -frames 0 \Q$path};
+            die "no longer works with mpv";
 
-            $w = $mplayer =~ /^ID_VIDEO_WIDTH=(\d+)$/sm ? $1 : undef;
-            $h = $mplayer =~ /^ID_VIDEO_HEIGHT=(\d+)$/sm ? $1 : undef;
+            my $mpv = qx{LC_ALL=C exec \Q$Gtk2::CV::MPV\E </dev/null 2>/dev/null -really-quiet -noconfig all -noautosub -nofontconfig -sub /dev/null -sub-fuzziness 0 -cache-min 0 -input nodefault-bindings:conf=/dev/null -identify -vo null -ao null -frames 0 \Q$path};
 
-            if ($w && $mplayer =~ /^ID_VIDEO_ASPECT=([0-9\.]+)$/sm && $1 > 0) {
+            $w = $mpv =~ /^ID_VIDEO_WIDTH=(\d+)$/sm ? $1 : undef;
+            $h = $mpv =~ /^ID_VIDEO_HEIGHT=(\d+)$/sm ? $1 : undef;
+
+            if ($w && $mpv =~ /^ID_VIDEO_ASPECT=([0-9\.]+)$/sm && $1 > 0) {
                $w = POSIX::ceil $w * $1 * ($h / $w); # correct aspect ratio, assume square pixels
             } else {
-               # no idea what to do, mplayer's aspect factors seem to be random
+               # no idea what to do, mpv's aspect factors seem to be random
                #$w = POSIX::ceil $w * 1.50 * ($h / $w); # correct aspect ratio, assume square pixels
                #$w = POSIX::ceil $w * 1.33;
             }
@@ -478,8 +504,8 @@ sub load_image {
             # the window to have the correct size
             $self->show;
 
-            # add a couple of windows just for mplayer's sake
-            my $box = $self->{mplayer_box} = new Gtk2::EventBox;
+            # add a couple of windows just for mpv's sake
+            my $box = $self->{mpv_box} = new Gtk2::EventBox;
             $box->set_above_child (1);
             $box->set_visible_window (0);
             $box->set_events ([]);
@@ -491,12 +517,12 @@ sub load_image {
             $box->show_all;
             $window->realize;
 
-            $self->{mplayer_window} = $window;
+            $self->{mpv_window} = $window;
 
             my $xid = $window->window->get_xid;
 
             socketpair my $sfh, my $mfh, Socket::AF_UNIX (), Socket::SOCK_STREAM (), 0;
-            $self->{mplayer_fh} = $mfh;
+            $self->{mpv_fh} = $mfh;
             $mfh->autoflush (1);
             fcntl $mfh, Fcntl::F_SETFD (), Fcntl::FD_CLOEXEC ();
             fcntl $mfh, Fcntl::F_SETFL (), Fcntl::O_NONBLOCK ();
@@ -513,30 +539,32 @@ sub load_image {
                   ["-vf-add", "rotate=2"],
                );
 
-               my @mplayer_args;
+               my @mpv_args;
 
-               if ($Gtk2::CV::MPLAYER_IS_MPV) {
-                  @mplayer_args = (
-                     qw(
-                        --no-input-terminal --no-input-default-bindings --no-input-cursor --input-conf=/dev/null
-                        --input-vo-keyboard=no -loop-file=inf
-                     ),
-                     @extra_mpv_args,
-                     "--input-file=fd://" . fileno $sfh
-                  );
-                  open STDIN, "</dev/null";
-                  open STDOUT, ">/dev/null";
-                  fcntl $sfh, Fcntl::F_SETFD (), 0;
+               @mpv_args = (
+                  qw(
+                     --no-terminal --no-input-terminal --no-input-default-bindings --no-input-cursor --input-conf=/dev/null
+                     --input-vo-keyboard=no -loop-file=inf
+                  ),
+                  @extra_mpv_args,
+               );
+
+               $Gtk2::CV::mpv_list_options //= qx<$Gtk2::CV::MPV --list-options>;
+
+               if ($Gtk2::CV::mpv_list_options =~ /^--input-ipc-file\s/m) {
+                  push @mpv_args, "--input-ipc-file=fd://" . fileno $sfh;
                } else {
-                  @mplayer_args = qw(-nofs -slave -fixed-vo -nokeepaspect -input nodefault-bindings:conf=/dev/null -zoom -loop 0);
-                  open STDIN, "+<&" . fileno $sfh;
-                  open STDOUT, ">&" . fileno $sfh;
-                  # $sfh will autoclose
+                  # we always assume --input-file, as list-options doens't list it even if it is there
+                  push @mpv_args, "--input-file=fd://" . fileno $sfh;
                }
 
+               open STDIN, "</dev/null";
+               open STDOUT, ">/dev/null";
+               fcntl $sfh, Fcntl::F_SETFD (), 0;
+
                #open STDERR, ">/dev/null";
 
-               exec $Gtk2::CV::MPLAYER, @mplayer_args, qw(-quiet),
+               exec $Gtk2::CV::MPV, @mpv_args, qw(-quiet),
                                @{ $rotate[$self->{load_rotate} / 90] },
                                -wid => $xid, $path;
                POSIX::_exit 0;
@@ -556,22 +584,11 @@ sub load_image {
                      my $line = $1;
 
                      if ($line =~ /^\{/) {
-                        # mpv
-
                         $line = eval { JSON::XS::decode_json $line };
 
                         if ($line->{event} eq "video-reconfig") {
-                           $self->update_mplayer_window;
+                           $self->update_mpv_window;
                         }
-
-                     } else {
-                        # mplayer
-
-                        if ($line =~ /ANS_FILENAME=/) {
-                           # presumably, everything is set-up now
-                           $self->update_mplayer_window;
-                        }
-
                      }
                   }
                } elsif (defined $len or $! != Errno::EAGAIN) {
@@ -864,15 +881,15 @@ sub crop {
    );
 }
 
-sub update_mplayer_window {
+sub update_mpv_window {
    my ($self) = @_;
 
-   # force a resize of the mplayer window, otherwise it doesn't receive
+   # force a resize of the mpv window, otherwise it doesn't receive
    # a configureevent :/
-   $self->{mplayer_window}->window->resize (1, 1),
-   $self->{mplayer_window}->window->resize ($self->{w}, $self->{h})
-      if $self->{mplayer_window}
-         && $self->{mplayer_window}->window;
+   $self->{mpv_window}->window->resize (1, 1),
+   $self->{mpv_window}->window->resize ($self->{w}, $self->{h})
+      if $self->{mpv_window}
+         && $self->{mpv_window}->window;
 }
 
 sub do_configure {
@@ -888,7 +905,7 @@ sub do_configure {
    $self->{w} = $w;
    $self->{h} = $h;
 
-   $self->update_mplayer_window;
+   $self->update_mpv_window;
 
    return unless $self->{subimage};
 
@@ -906,9 +923,9 @@ sub do_configure {
 sub handle_key {
    my ($self, $key, $state) = @_;
 
-   local $SIG{PIPE} = 'IGNORE'; # for mplayer_fh
+   local $SIG{PIPE} = 'IGNORE'; # for mpv_fh
 
-   my $volume = $Gtk2::CV::MPLAYER_IS_MPV ? "add ao-volume %s2" : "volume %s1";
+   my $volume = "add ao-volume %s2";
 
    $state *= Gtk2::Accelerator->get_default_mod_mask;
 
@@ -954,10 +971,10 @@ sub handle_key {
          $self->resize ($self->{dw} * 2, $self->{dh} * 2);
 
       } elsif ($key == $Gtk2::Gdk::Keysyms{comma}) {
-         $self->resize ($self->{dw} * 0.9, $self->{dh} * 0.9);
+         $self->resize ($self->{dw} / 1.1 + 0.5, $self->{dh} / 1.1 + 0.5);
 
       } elsif ($key == $Gtk2::Gdk::Keysyms{period}) {
-         $self->resize ($self->{dw} * 1.1, $self->{dh} * 1.1);
+         $self->resize ($self->{dw} * 1.1      , $self->{dh} * 1.1);
 
       } elsif ($key == $Gtk2::Gdk::Keysyms{n}) {
          $self->auto_resize;
@@ -979,10 +996,6 @@ sub handle_key {
          $self->{interp} = 'bilinear';
          $self->force_redraw; $self->redraw;
 
-      } elsif ($key == $Gtk2::Gdk::Keysyms{S}) {
-         $self->{interp} = 'hyper';
-         $self->force_redraw; $self->redraw;
-
       } elsif ($key == $Gtk2::Gdk::Keysyms{t}) {
          my $r = $::REVERSE_ROTATION ? +90 : -90;
          $self->set_subimage (Gtk2::CV::rotate $self->{subimage}, $r);
@@ -1020,35 +1033,35 @@ sub handle_key {
 
          delete $self->{drag_info};
 
-      # extra mplayer controls
+      # extra mpv controls
       } elsif ($self->{player_pid} && $key == $Gtk2::Gdk::Keysyms{Right}) {
-         print {$self->{mplayer_fh}} "seek +10\n";
+         print {$self->{mpv_fh}} "seek +10\n";
       } elsif ($self->{player_pid} && $key == $Gtk2::Gdk::Keysyms{Left}) {
-         print {$self->{mplayer_fh}} "seek -10\n";
+         print {$self->{mpv_fh}} "seek -10\n";
       } elsif ($self->{player_pid} && $key == $Gtk2::Gdk::Keysyms{Up}) {
-         print {$self->{mplayer_fh}} "seek +60\n";
+         print {$self->{mpv_fh}} "seek +60\n";
       } elsif ($self->{player_pid} && $key == $Gtk2::Gdk::Keysyms{Down}) {
-         print {$self->{mplayer_fh}} "seek -60\n";
+         print {$self->{mpv_fh}} "seek -60\n";
       } elsif ($self->{player_pid} && $key == $Gtk2::Gdk::Keysyms{Page_Up}) {
-         print {$self->{mplayer_fh}} "seek +600\n";
+         print {$self->{mpv_fh}} "seek +600\n";
       } elsif ($self->{player_pid} && $key == $Gtk2::Gdk::Keysyms{Page_Down}) {
-         print {$self->{mplayer_fh}} "seek -600\n";
+         print {$self->{mpv_fh}} "seek -600\n";
       } elsif ($self->{player_pid} && $key == $Gtk2::Gdk::Keysyms{'#'}) {
-         print {$self->{mplayer_fh}} "switch_audio\n";
+         print {$self->{mpv_fh}} "switch_audio\n";
       } elsif ($self->{player_pid} && $key == $Gtk2::Gdk::Keysyms{j}) {
-         print {$self->{mplayer_fh}} "sub_select\n";
+         print {$self->{mpv_fh}} "sub_select\n";
       } elsif ($self->{player_pid} && $key == $Gtk2::Gdk::Keysyms{o}) {
-         print {$self->{mplayer_fh}} "no-osd cycle-values osd-level 2 3 0 1\n";
+         print {$self->{mpv_fh}} "no-osd cycle-values osd-level 2 3 0 1\n";
       } elsif ($self->{player_pid} && $key == $Gtk2::Gdk::Keysyms{p}) {
-         print {$self->{mplayer_fh}} "cycle pause\n";
+         print {$self->{mpv_fh}} "cycle pause\n";
       } elsif ($self->{player_pid} && $key == $Gtk2::Gdk::Keysyms{Escape}) {
-         print {$self->{mplayer_fh}} "quit\n";
+         print {$self->{mpv_fh}} "quit\n";
       } elsif ($self->{player_pid} && $key == $Gtk2::Gdk::Keysyms{9}) {
-         printf {$self->{mplayer_fh}} "$volume\n", "-";
+         printf {$self->{mpv_fh}} "$volume\n", "-";
       } elsif ($self->{player_pid} && $key == $Gtk2::Gdk::Keysyms{0}) {
-         printf {$self->{mplayer_fh}} "$volume\n", "+";
+         printf {$self->{mpv_fh}} "$volume\n", "+";
 #      } elsif ($self->{player_pid} && $key == $Gtk2::Gdk::Keysyms{f}) {
-#         print {$self->{mplayer_fh}} "vo_fullscreen\n";
+#         print {$self->{mpv_fh}} "vo_fullscreen\n";
 
       } else {
 
