@@ -53,9 +53,16 @@ sub INIT_INSTANCE {
       $k = $self->{cluster}{$k};
 
       $self->{updating} = 1;
-      $self->{schnauzer}->set_paths ($k, 0, sub {
-         delete $self->{updating};
-      });
+
+      if ($self->{dirview}) {
+         $self->{schnauzer}->chdir ($k->[0], sub {
+            delete $self->{updating};
+         });
+      } else {
+         $self->{schnauzer}->set_paths ($k, 0, sub {
+            delete $self->{updating};
+         });
+      }
 
       1
    });
@@ -72,80 +79,104 @@ sub INIT_INSTANCE {
 }
 
 sub analyse {
-   my ($self) = @_;
+   my ($self, $clustersise) = @_;
 
    $self->{schnauzer}->push_state;
 
-   my $progress = new Gtk2::CV::Progress title => "splitting...";
-
    my $paths = $self->{schnauzer}->get_paths;
    $self->{select} = [$paths];
 
-   my %files = map {
-                      my $orig = $_;
-                      s/^.*\///;
-                      s/(?:-\d+|\.~[~0-9]*)+$//; # remove -000, .~1~ etc.
-                      s/\.[^.]*$//;
-                      ($orig => [/(\pL(?:\pL+|\pP(?=\pL))* | \pN+)/gx])
-                   }
-                   grep !/\.(sfv|crc|par|par2)$/i,
-                        @{ $self->{select}[-1] };
+   my ($progress, %groups);
 
-   my $cluster = ();
+   if ($self->{dirview}) {
 
-   $progress->update (0.25);
-   $progress->set_title ("clustering...");
+      $progress = new Gtk2::CV::Progress title => "preparing...";
 
-   for my $regex (
-      qr/^\PN/,
-      qr/^\pN/,
-   ) {
-      my %c;
-      while (my ($k, $v) = each %files) {
-         my $idx = "aaaaaa";
-         # clusterise by component_idx . component
-         push @{ $c{$idx++ . $_} }, $k
-            for grep m/$regex/, @$v;
+      for my $entry (@{ $self->{schnauzer}{entry} }) {
+         # skip nondirs
+         $entry->[Gtk2::CV::Schnauzer::E_FLAGS] & Gtk2::CV::Schnauzer::F_ISDIR
+            or next;
+
+         my $dir  = $entry->[Gtk2::CV::Schnauzer::E_DIR ];
+         my $file = $entry->[Gtk2::CV::Schnauzer::E_FILE];
+
+         $groups{$file} = ["$dir/$file"];
       }
 
-      $cluster = { %c, %$cluster };
+   } else {
 
-      delete @files{ @{ $c{$_} } }
-         for grep @{ $c{$_} } >= 3, keys %c;
-   }
+      $progress = new Gtk2::CV::Progress title => "splitting...";
 
-   $progress->update (0.5);
-   $progress->set_title ("categorize...");
+      my %files = map {
+                         my $orig = $_;
+                         s/^.*\///;
+                         s/(?:-\d+|\.~[~0-9]*)+$//; # remove -000, .~1~ etc.
+                         s/\.[^.]*$//;
+                         ($orig => [/(\pL(?:\pL+|\pP(?=\pL))* | \pN+)/gx])
+                      }
+                      grep !/\.(sfv|crc|par|par2)$/i,
+                           @{ $self->{select}[-1] };
 
-   $cluster->{"000000REMAINING FILES"} = [keys %files];
+      my $cluster = ();
 
-   # remove component index
-   my %clean;
-   while (my ($k, $v) = each %$cluster) {
-      if (exists $clean{substr $k, 6}) {
-         my $idx = 0;
-         ++$idx while exists $clean{(substr $k, 6) . "/$idx"};
-         $k .= "/$idx";
+      $progress->update (0.25);
+      $progress->set_title ("clustering...");
+
+      for my $regex (
+         qr/^\PN/,
+         qr/^\pN/,
+      ) {
+         my %c;
+         while (my ($k, $v) = each %files) {
+            my $idx = "aaaaaa";
+            # clusterise by component_idx . component
+            push @{ $c{$idx++ . $_} }, $k
+               for grep m/$regex/, @$v;
+         }
+
+         $cluster = { %c, %$cluster };
+
+         delete @files{ @{ $c{$_} } }
+            for grep @{ $c{$_} } >= 3, keys %c;
       }
-      $clean{substr $k, 6} = $v;
-   }
 
-   $self->{cluster} = \%clean;
+      $progress->update (0.5);
+      $progress->set_title ("categorize...");
+
+      $cluster->{"000000REMAINING FILES"} = [keys %files];
+
+      # remove component index
+      while (my ($k, $v) = each %$cluster) {
+         if (exists $groups{substr $k, 6}) {
+            my $idx = 0;
+            ++$idx while exists $groups{(substr $k, 6) . "/$idx"};
+            $k .= "/$idx";
+         }
+         $groups{substr $k, 6} = $v;
+      }
+
+      while (my ($k, $v) = each %groups) {
+         delete $groups{$k}
+            unless @$v > 1;
+      }
+   }
 
    $progress->update (0.75);
    $progress->set_title ("finishing...");
 
+   $self->{cluster} = \%groups;
+
    @{ $self->{list}{data} } = (
       sort { $b->[0] <=> $a->[0] }
-         grep $_->[0] > 1,
-              map [(scalar @{ $self->{cluster}{$_} }), $_], keys %{ $self->{cluster} }
+         map [(scalar @{ $groups{$_} }), $_], sort keys %groups
    );
 }
 
 sub start {
-   my ($self, $schnauzer) = @_;
+   my ($self, $schnauzer, $dirview) = @_;
 
    $self->{schnauzer} = $schnauzer;
+   $self->{dirview} = $dirview;
 
    $self->{signal} = $schnauzer->signal_connect_after (chpaths => sub {
       return if $self->{updating};
@@ -171,7 +202,13 @@ sub new_schnauzer {
       $menu->append (my $i_up = new Gtk2::MenuItem "Filename clustering...");
       $i_up->signal_connect (activate => sub {
          $namecluster = Gtk2::CV::Plugin::NameCluster->new;
-         $namecluster->start ($self);
+         $namecluster->start ($self, 0);
+      });
+
+      $menu->append (my $i_up = new Gtk2::MenuItem "Subdir view...");
+      $i_up->signal_connect (activate => sub {
+         $namecluster = Gtk2::CV::Plugin::NameCluster->new;
+         $namecluster->start ($self, 1);
       });
    });
 
@@ -187,13 +224,11 @@ sub new_schnauzer {
          $state == ["control-mask"]
          && ($key == $Gtk2::Gdk::Keysyms{Up} || $key == $Gtk2::Gdk::Keysyms{Down})
       ) {
-         if ($namecluster) {
+         if ($namecluster && $namecluster->{list}) {
             my ($path) = $namecluster->{list}->get_cursor;
             $key == $Gtk2::Gdk::Keysyms{Up} ? $path->prev : $path->next;
             $namecluster->{list}->set_cursor ($path);
             return 1;
-         } else {
-            return 1;
          }
       }
 
