@@ -24,7 +24,10 @@ package Gtk2::CV::Schnauzer;
 use common::sense;
 use integer;
 
-our %EXCLUDE; # to be set form your .cvrc to exclude additional files
+our %EXCLUDE; # to be set from your .cvrc to exclude additional files
+our $ICONSCALE; # to be set from your .cvrc to set icon scale
+our $FONTSCALE; # to be set from your .cvrc to set icon font scale
+our $DISPLAYSCALE; # to be set from your .cvrc to set display scale (for hidpi displays)
 
 use Gtk2;
 use Gtk2::Pango;
@@ -99,6 +102,11 @@ sub shellquote_selection($) {
    }
 }
 
+sub format_size($) {
+   # perl doesn't support %'d
+   scalar reverse join ",", unpack "(a3)*", scalar reverse "$_[0]"
+}
+
 sub regdir($) {
    $dir{$_[0]} ||= ++$dirid;
 }
@@ -116,16 +124,37 @@ sub parent_dir($) {
 
 sub IW() { 80 } # must be the same as in CV.xs(!)
 sub IH() { 60 } # must be the same as in CV.xs(!)
-sub IX() { 20 }
-sub IY() { 16 }
-sub FY() { 12 } # font-y
+
+sub default_display_scale {
+   init Gtk2::Gdk;
+
+   my $screen = Gtk2::Gdk::Screen->get_default
+      or return 1;
+
+   my $rect = $screen->get_monitor_geometry (0)
+      or return 1;
+
+   no integer;
+
+   my $dpi = $rect->width * 2.54 * 10 / $screen->get_monitor_width_mm (0);
+
+   $dpi < 110 ? 1 : $dpi / 96
+}
 
 BEGIN {
    no integer;
-   my $scale = $ENV{CV_THUMBNAIL_SCALE} || 1;
-   eval "sub ISCALE() { $scale }";
-   eval "sub IWD()    { IW * $scale }";
-   eval "sub IHD()    { IH * $scale }";
+   $ICONSCALE = $ENV{CV_THUMBNAIL_SCALE} || $ICONSCALE || 1;
+   eval "sub IWG()    { " . (int IW * $ICONSCALE   ) . " }"; # "G"eneration
+   eval "sub IHG()    { " . (int IH * $ICONSCALE   ) . " }"; # "G"eneration
+
+   $DISPLAYSCALE = $ENV{CV_DISPLAY_SCALE} || $DISPLAYSCALE || default_display_scale;
+   eval "sub IWD()    { " . (int IW * $DISPLAYSCALE) . " }"; # "D"isplay
+   eval "sub IHD()    { " . (int IH * $DISPLAYSCALE) . " }"; # "D"isplay
+   eval "sub IX()     { " . (int 20 * $DISPLAYSCALE) . " }"; # extra horizontal space
+
+   my $fsize = int 12 * ($ENV{CV_SCHNAUZER_FONTSCALE} || $FONTSCALE || $ICONSCALE) * $DISPLAYSCALE;
+   eval "sub FY()     { $fsize }";
+   eval "sub IY()     { " . (ceil 1.25 * $fsize) . " }";
 }
 
 sub SCROLL_Y()    {  1 }
@@ -147,8 +176,14 @@ sub E_FILE  () { 1 }
 sub E_FLAGS () { 2 }
 sub E_PIXMAP() { 3 }
 
-sub img {
-   scalar +(Gtk2::CV::require_image $_[0])->render_pixmap_and_mask (0.5)
+sub load_icon {
+   my $pb = Gtk2::CV::require_image $_[0];
+
+   $pb->scale_simple (IWD, IHD, "bilinear")
+}
+
+sub load_icon_pm {
+   scalar +(load_icon $_[0])->render_pixmap_and_mask (0.5)
 }
 
 my %combine;
@@ -245,18 +280,18 @@ my $dirs_layer;
 my %file_img;
 
 sub init {
-   $file_img = img "file.png";
-   $diru_img = img "dir-unvisited.png";
-   $dirv_img = img "dir-visited.png";
+   $file_img = load_icon_pm "file.png";
+   $diru_img = load_icon_pm "dir-unvisited.png";
+   $dirv_img = load_icon_pm "dir-visited.png";
   
-   $dirx_layer = Gtk2::CV::require_image "dir-xvpics.png";
-   $dire_layer = Gtk2::CV::require_image "dir-empty.png";
-   $dirs_layer = Gtk2::CV::require_image "dir-symlink.png";
+   $dirx_layer = load_icon "dir-xvpics.png";
+   $dire_layer = load_icon "dir-empty.png";
+   $dirs_layer = load_icon "dir-symlink.png";
   
    %file_img = do {
       my %logo = reverse %ext_logo;
 
-      map +($_ => img "file-$_.png"), keys %logo
+      map +($_ => load_icon_pm "file-$_.png"), keys %logo
    };
 }
 
@@ -354,9 +389,10 @@ sub video_thumbnail_at {
       my $dir = File::Temp::tempdir DIR => $Gtk2::CV::FAST_TMP
          or return;
 
-      system "exec mplayer -really-quiet -noautosub -nosound -input nodefault-bindings -noconfig all -ss $time -frames 2 -vf scale=" . IW . ":-2::::::1 -vo pnm:outdir=\Q$dir\E \Q$path\E >/dev/null 2>&1 </dev/null";
+      #system "exec mpv -really-quiet -noautosub -nosound -input nodefault-bindings -noconfig all -ss $time -frames 2 -vf scale=" . IWG . ":-2::::::1 -vo pnm:outdir=\Q$dir\E \Q$path\E >/dev/null 2>&1 </dev/null";
+      system "exec mpv --no-terminal --sid=no --ao=null --no-config --start=$time --frames=2 --vf=scale=" . IWG . ":-2::::::1 --vo=image -vo-image-format=png --vo-image-png-compression=0 -vo-image-outdir=\Q$dir\E \Q$path\E >/dev/null 2>&1 </dev/null";
 
-      my $pb = eval { new_from_file Gtk2::Gdk::Pixbuf "$dir/00000002.ppm" };
+      my $pb = eval { new_from_file Gtk2::Gdk::Pixbuf "$dir/00000002.png" };
 
       # mplayer can write a large number of files when a single one is requested...
       if (opendir my $fh, $dir) {
@@ -423,13 +459,14 @@ sub {
          if (IO::AIO::mmap my $data, -s $fh, IO::AIO::PROT_READ, IO::AIO::MAP_SHARED, $fh) {
             my $type = Gtk2::CV::filetype $data;
 
-            $pb = eval { $type eq "jpg" && Gtk2::CV::decode_jpeg $data, 1, IWD, IHD }
-                  || eval { $type eq "webp" && Gtk2::CV::decode_webp $data, 1, IWD, IHD }
+            $pb = eval { $type eq "image/jpeg" && Gtk2::CV::decode_jpeg $data, 1, IWG, IHG }
+                  || eval { $type eq "image/jxl" && Gtk2::CV::decode_jxl $data, 1, IWG, IHG }
+                  || eval { $type eq "image/webp" && Gtk2::CV::decode_webp $data, 1, IWG, IHG }
                   || eval {
                         my $loader = new Gtk2::Gdk::PixbufLoader;
 
                         # should set size-prepared callback to scale down, but
-                        # we only really care about this for jpegs, which are handled above.
+                        # we only really care about this for jpeg files, which are handled above.
                         #$loader->set_size (IW, IH);
 
                         $loader->write ($data);
@@ -448,12 +485,12 @@ sub {
 
       my ($w, $h) = ($pb->get_width, $pb->get_height);
 
-      if ($w * IHD > $h * IWD) {
-         $h = (int $h * IWD / $w + 0.5) || 1;
-         $w = IWD;
+      if ($w * IHG > $h * IWG) {
+         $h = (int $h * IWG / $w + 0.5) || 1;
+         $w = IWG;
       } else {
-         $w = (int $w * IHD / $h + 0.5) || 1;
-         $h = IHD;
+         $w = (int $w * IHG / $h + 0.5) || 1;
+         $h = IHG;
       }
 
       $pb = Gtk2::CV::dealpha $pb->scale_simple ($w, $h, 'tiles');
@@ -1117,9 +1154,9 @@ sub emit_sel_changed {
          aio_stat "$entry->[E_DIR]/$entry->[E_FILE]", sub {
             return unless $id == $self->{aio_sel_changed};
             $self->{info}->set_text (
-               sprintf "%s: %d bytes, last modified %s (in %s)",
+               sprintf "%s: %s bytes, last modified %s (in %s)",
                        (filename_display_name $entry->[E_FILE]),
-                       -s _,
+                       (format_size -s _),
                        (strftime "%Y-%m-%d %H:%M:%S", localtime +(stat _)[9]),
                        (filename_display_name $entry->[E_DIR]),
             );
@@ -1314,7 +1351,7 @@ sub cursor_move {
       $self->clear_selection;
       $cursor_prev = -1;
 
-      if (!$cursor) {
+      unless (defined $cursor) {
          if ($inc < 0) {
             $cursor = $self->{offs} + $self->{page} * $self->{cols} - 1;
          } else {
